import os, sys, json
from dotenv import load_dotenv

load_dotenv()

DIR_NAME = os.path.dirname(os.path.abspath(__file__))
root = os.path.dirname(os.path.dirname(DIR_NAME))
sys.path.append(root)

import ttxt


def getTtxtExchange(exchangeName):
    if exchangeName == "gateFutures":
        print("returned gateFuture")
        return ttxt.gateFutures(key=os.getenv("KEY"), secret=os.getenv("SECRET"))
    if exchangeName == "bybitFutures":
        print("returned bybitFutures")
        return ttxt.bybitFutures(key=os.getenv("KEY"), secret=os.getenv("SECRET"))
    if exchangeName == "bingx":
        print("returned bybitFutures")
        return ttxt.bingx(key=os.getenv("KEY"), secret=os.getenv("SECRET"))
    if exchangeName == "biconomy":
        print("returned biconomy")
        return ttxt.biconomy(key=os.getenv("KEY"), secret=os.getenv("SECRET"))
    else:
        print(f"Exchange not supported: {exchangeName}")

def test(exchangeName="", params={}):
    ttxtExchange = getTtxtExchange(exchangeName)
    # ttxtExchange = ttxt.gateFutures(key=os.getenv("KEY"), secret=os.getenv("SECRET"))

    if "ticker" in params and params["ticker"]:
        print("testing ticker...")
        try:
            resp = ttxtExchange.fetch_ticker(tickerToTest)
            print(resp)
        except Exception as e:
            print("ticker could not be fetched")
            print(e)
    if "balance" in params and params["balance"]:
        print("testing balance...")
        try:
            resp = ttxtExchange.fetch_balance()
            print(resp)
        except Exception as e:
            print("balance could not be fetched")
            print(e)
    if "createOrder" in params and params["createOrder"]:
        print("testing create order...")
        try:
            resp = ttxtExchange.create_order(symbol=tickerToTest, type="limit", amount="0.001", side="buy", price=41000)
            print(f"createOrder response: {resp}")
        except Exception as e:
            print("order could not be created")
            print(e)
    if "createLimitOrder" in params and params["createLimitOrder"]:
        print("testing create order...")
        try:
            resp = ttxtExchange.create_limit_order(symbol=tickerToTest, amount=0.0015436, side="buy", price=41900.90878)
            print(f"createLimitOrder response: {resp}")
        except Exception as e:
            print("limit order could not be created")
            print(e)
    if "createMarketSellOrder" in params and params["createMarketSellOrder"]:
        print("testing create order...")
        try:
            resp = ttxtExchange.create_market_sell_order(symbol=tickerToTest, amount=0.001)
            print(f"createMarketSellOrder response: {resp}")
        except Exception as e:
            print("market sell order could not be created")
            print(e)
    if "createMarketBuyOrder" in params and params["createMarketBuyOrder"]:
        print("testing create order...")
        try:
            resp = ttxtExchange.create_market_buy_order(symbol=tickerToTest, amount=0.001)
            print(f"createMarketBuyOrder response: {resp}")
        except Exception as e:
            print("market buy order could not be created")
            print(e)
    if "fetchOrder" in params and params["fetchOrder"]:
        print("testing fetch order...")
        try:
            resp = ttxtExchange.fetch_order(id=45577069671, symbol=tickerToTest)
            print(f"fetchOrder response: {resp}")
        except Exception as e:
            print("order could not be fetched")
            print(e)
    if "cancelOrder" in params and params["cancelOrder"]:
        print("testing cancel order...")
        try:
            resp = ttxtExchange.cancel_order(id=45576968750, symbol=tickerToTest)
            print(f"cancelOrder response: {resp}")
        except Exception as e:
            print("order could not be canceled")
            print(e)
    if "fetchOpenOrders" in params and params["fetchOpenOrders"]:
        print("testing open orders...")
        try:
            resp = ttxtExchange.fetch_open_orders(symbol=tickerToTest)
            print(f"fetchOpenOrders response: {resp}")
        except Exception as e:
            print("OpenOrders could not be fetched")
            print(e)
    if "setLeverage" in params and params["setLeverage"]:
        print("testing set leverage...")
        try:
            resp = ttxtExchange.set_leverage(symbol=tickerToTest, leverage=10)
            print(f"set leverage response: {resp}")
        except Exception as e:
            print("leverage could not be set")
            print(e)
    if "ohlcv" in params and params["ohlcv"]:
        print("testing ohlcv...")
        try:
            resp = ttxtExchange.fetch_ohlcv(symbol=tickerToTest, interval="hour", size=10)
            print(f"ohlcv response: {resp}")
        except Exception as e:
            print("leverage could not be set")
            print(e)
    if "orderbook" in params and params["orderbook"]:
        print("testing orderbook...")
        try:
            resp = ttxtExchange.fetch_order_book(symbol=tickerToTest, limit=100)
            print(f"orderbook response: {resp}")
        except Exception as e:
            print("orderbook could not be fetched")
            print(e)
    if "candlestick" in params and params["candlestick"]:
        print("testing candlestick...")
        try:
            resp = ttxtExchange.fetch_ohlcv(symbol=tickerToTest, timeframe='1h', limit=1)
            print(f"candlestick response: {resp}")
            print(f"candlestick response len: {len(resp)}")
        except Exception as e:
            print("candlesticks could not be fetched")
            print(e)


if __name__ == "__main__":
    tickerToTest = "GALA/USDT" #True
    testingParams = {"ticker": False, "balance": False, "ohlcv": False, "createOrder": False, "fetchOrder": False, 
                     "cancelOrder": False, "fetchOpenOrders": False, "setLeverage": False, "createLimitOrder": False,
                     "createMarketSellOrder": False, "createMarketBuyOrder": False, "orderbook": False, "candlestick": True}
    test(params=testingParams, exchangeName="gateFutures")